#!/usr/bin/perl

use strict;
use warnings;

#
# monomial is a list of length L <= N*(N-1)/2 listing the powers of the
# variables; e.g., (1, 2, 0, ..., 0) <=> x1*x2*x2; total degree is just
# the sum of the elements
#
# permutation initially acts on the atoms and so is represented by a list
# of length N : (0, 1, 2, 3, ..., N) is "identity", and so on
#
# for (H2O)x2 the atoms are: Oa Ha1 Ha2 Ob Hb1 Hb2
# and possible permutations permute Hydrogens withing the molecules
# and molecules themselves
#

my $i0 = 0;
my $nmolec = 2;
my @atoms = qw(Oa Ha1 Ha2 Ob Hb1 Hb2);
my %atom_idx;
for (my $n = 0; $n < scalar @atoms; ++$n) {
    $atom_idx{$atoms[$n]} = $n;
}

my @permutations;
my %h_perm;

my @hmap = ("1:2","1:4","1:5","2:4","2:5","4:5");
my @omap = ("0:3");

foreach my $h12 (0, 1) {
    foreach my $h13 (0, 1) {
    foreach my $h14 (0, 1) {
    foreach my $h23 (0, 1) {
    foreach my $h24 (0, 1) {
    foreach my $h34 (0, 1) {
#    foreach my $o56 (0, 1) {
    my $o56 = 0;
	my @h_mask = ($h12, $h13, $h14, $h23, $h24, $h34);
	my @o_mask = ($o56);
	my $tmp = &permute_all(\@h_mask, \@hmap, \@o_mask, \@omap);
	$h_perm{join(':', @{$tmp})} = undef;
#    }
    }
    }
    }
    }
    }
}

@permutations = keys %h_perm;

print STDERR scalar @permutations, " permutations:\n";
foreach (@permutations) {
    my @toks = split(/:/,$_);
    my @arr = ();
    $_ = \@arr;
    for (my $n = 0; $n < scalar @atoms; ++$n) {
	push @arr, $atom_idx{$toks[$n]};
    }

    print STDERR join(' ', @{$_}), "\n";
}

#
# now find how the permutations act on the monomials
# composed out of (some) distances
#

my @variables = (['Ha1', 'Ha2'],
                 ['Ha1', 'Hb1'],
                 ['Ha1', 'Hb2'],
                 ['Ha2', 'Hb1'],
                 ['Ha2', 'Hb2'],
                 ['Hb1', 'Hb2'],
                 ['Oa', 'Ha1'],
                 ['Oa', 'Ha2'],
                 ['Oa', 'Hb1'],
                 ['Oa', 'Hb2'],
                 ['Ob', 'Ha1'],
                 ['Ob', 'Ha2'],
                 ['Ob', 'Hb1'],
                 ['Ob', 'Hb2'],
                 ['Oa', 'Ob']);

print STDERR "\n";
my $nvar = scalar @variables;
my %atom_pair_to_var;
for (my $v = 0; $v < $nvar; ++$v) {
    my $key = join(':', sort @{$variables[$v]});
    print STDERR $key, "\n";
    $atom_pair_to_var{$key} = $v;
}
print STDERR "\n";

#
# now loop over 1st degree monomials [we want 2B-terms only]
#

my %mono_all;
for (my $k1 = 0; $k1 < $nvar; ++$k1) {
        my @mono = (0) x $nvar;
        $mono[$k1] += 1;
        next unless is_2B(\@mono);
#        print STDERR join(':', @mono), "\n";
        $mono_all{join(':', @mono)} = undef;
}

#print STDERR "\n";
print STDERR scalar keys %mono_all, " possible 2B, 1st degree monomials\n";
my $mono1_orb = find_orbits(\%mono_all);

print STDERR scalar @{$mono1_orb}, " 1st degree terms\n";
#
# now loop over 2nd degree monomials [we want 2B-terms only]
#

for (my $k1 = 0; $k1 < $nvar; ++$k1) {
    for (my $k2 = 0; $k2 < $nvar; ++$k2) {
        my @mono = (0) x $nvar;
        $mono[$k1] += 1;
        $mono[$k2] += 1;
        next unless is_2B(\@mono);
#        print STDERR join(':', @mono), "\n";
        $mono_all{join(':', @mono)} = undef;
    }
}

#print STDERR "\n";
print STDERR scalar keys %mono_all, " possible 2B, 2nd degree monomials\n";
my $mono2_orb = find_orbits(\%mono_all);

print STDERR scalar @{$mono2_orb}, " 2nd degree terms\n";

#
# now loop over 3rd degree monomials [keeping 2B-terms only]
#

for (my $k1 = 0; $k1 < $nvar; ++$k1) {
    for (my $k2 = 0; $k2 < $nvar; ++$k2) {
        for (my $k3 = 0; $k3 < $nvar; ++$k3) {
            my @mono = (0) x $nvar;
            $mono[$k1] += 1;
            $mono[$k2] += 1;
            $mono[$k3] += 1;
            next unless is_2B(\@mono);
            $mono_all{join(':', @mono)} = undef;
        }
    }
}

print STDERR scalar keys %mono_all, " of possible 2B, 3rd degree monomials\n";
my $mono3_orb = find_orbits(\%mono_all);

print STDERR scalar @{$mono3_orb}, " 3rd degree terms\n";

#
# now loop over 4th degree monomials [keeping 2B-terms only]
#

for (my $k1 = 0; $k1 < $nvar; ++$k1) {
    for (my $k2 = 0; $k2 < $nvar; ++$k2) {
        for (my $k3 = 0; $k3 < $nvar; ++$k3) {
        for (my $k4 = 0; $k4 < $nvar; ++$k4) {
            my @mono = (0) x $nvar;
            $mono[$k1] += 1;
            $mono[$k2] += 1;
            $mono[$k3] += 1;
            $mono[$k4] += 1;
            next unless is_2B(\@mono);
            $mono_all{join(':', @mono)} = undef;
	}
        }
    }
}

print STDERR scalar keys %mono_all, " of possible 2B, 4th degree monomials\n";
my $mono4_orb = find_orbits(\%mono_all);

print STDERR scalar @{$mono4_orb}, " 4th degree terms\n";

#
# now loop over 5th degree monomials [keeping 2B-terms only]
#

for (my $k1 = 0; $k1 < $nvar; ++$k1) {
    for (my $k2 = 0; $k2 < $nvar; ++$k2) {
        for (my $k3 = 0; $k3 < $nvar; ++$k3) {
        for (my $k4 = 0; $k4 < $nvar; ++$k4) {
        for (my $k5 = 0; $k5 < $nvar; ++$k5) {
            my @mono = (0) x $nvar;
            $mono[$k1] += 1;
            $mono[$k2] += 1;
            $mono[$k3] += 1;
            $mono[$k4] += 1;
            $mono[$k5] += 1;
            next unless is_2B(\@mono);
            $mono_all{join(':', @mono)} = undef;
	}
	}
        }
    }
}

print STDERR scalar keys %mono_all, " of possible 2B, 5th degree monomials\n";
my $mono5_orb = find_orbits(\%mono_all);

print STDERR scalar @{$mono5_orb}, " 5th degree terms\n";

print "\n";
for (my $v = 0; $v < $nvar; ++$v) {
    print "    x[$v] = morse($variables[$v]->[0], $variables[$v]->[1]);\n"
}

print "\n\n";

my @orbs = ($mono1_orb, $mono2_orb, $mono3_orb, $mono4_orb, $mono5_orb);
#my @orbs = ($mono1_orb, $mono2_orb, $mono3_orb, $mono4_orb);
#my @orbs = ($mono2_orb, $mono3_orb);
#my @orbs = ($mono2_orb);

&print_poly("Oa", $i0, \@orbs, \@permutations);

    ## permute A <-> B ##

my @new_permutations = ();
&permute_two(0, 3, \@permutations, \@new_permutations);
&print_poly("Ob", $i0, \@orbs, \@new_permutations);

################################################################################

sub permute_all {
    my ($hmask, $hmap, $omask, $omap) = @_;

    my @dimer = qw(Oa Ha1 Ha2 Ob Hb1 Hb2);

    for (my $m = 0; $m < scalar @{$hmask}; ++$m) {
        if ($hmask->[$m] == 1) {
	    my @toks = split(/:/,$hmap->[$m]);
	    my $inda = $toks[0];
	    my $indb = $toks[1];

            my $tmp = $dimer[$inda];
            $dimer[$inda] = $dimer[$indb];
            $dimer[$indb] = $tmp;
        }
    }

    for (my $m = 0; $m < scalar @{$omask}; ++$m) {
        if ($omask->[$m] == 1) {
	    my @toks = split(/:/,$omap->[$m]);
	    my $inda = $toks[0];
	    my $indb = $toks[1];

            my $tmp = $dimer[$inda];
            $dimer[$inda] = $dimer[$indb];
            $dimer[$indb] = $tmp;
        }
    }

    return \@dimer;
}

################################################################################

sub is_2B {
    my $mono = shift;
    my %mols;

    die unless $nvar == scalar @{$mono};
    for (my $v = 0; $v < $nvar; ++$v) {
        next unless $mono->[$v] > 0;
        my $var = $variables[$v];
        my $m1 = substr ($var->[0], 1, 1);
        my $m2 = substr ($var->[1], 1, 1);
        $mols{$m1} = undef;
        $mols{$m2} = undef;
    }

    my $nmol = scalar keys %mols;

    return ($nmol == 2 ? 1 : 0);
}

################################################################################

sub find_orbits {
    my $all = shift;

    my @orbits;

    while (1) {
        my @still_there = keys %{$all};
        last if scalar @still_there == 0;
        my $curr = shift @still_there;
        push @orbits, $curr;
        for my $p (@permutations) {
            my $permuted = permute_monomial($curr, $p);
#	    print $count, " >> ", $curr, " | ", @{$p}, " | ", $permuted, "\n";
            delete ${$all}{$permuted};
        }
#	++$count;
    }

    return \@orbits;
}

################################################################################

sub permute_monomial {
    my ($src_str, $p) = @_;

    my @src = split(':', $src_str);
    my $nvar = scalar @src;
    my @dst = (0) x $nvar;

#    print $src_str, "\n", $p, "\n", @{$p}, "\n";
#    exit;
    # p is array ref for array of atoms numbers, e.g., 012345
    # src_str is distance combination (variables): 0:0:0:0:0:0:1:0:1:0:0:0:0:0:0

    for (my $v = 0; $v < $nvar; ++$v) {
        next unless $src[$v] > 0;
        my $atms = $variables[$v];
        my ($a1, $a2) = ($atom_idx{$atms->[0]}, $atom_idx{$atms->[1]});
#	print "$v : ", $src[$v], "  ", $atms->[0], "  $a1  ", $atms->[1], "  $a2  \n";
	# $a1 and $a2 are atom numbers, for Oa-Ob could be 0,3
	# if p = (3, x,x,0,x,x); where x is arbitrary, then 
	# which means @permuted = ("Ob", "Oa");
#	print "permute : ", $p->[$a1], " <-> ", $p->[$a2], "\n";
        my @permuted = ($atoms[$p->[$a1]], $atoms[$p->[$a2]]);
#	print @permuted, "\n";
        my $var = join(':', sort @permuted);
#	print $var, "\n";
        die " ** something is wrong ** :: @permuted\n"
            unless exists($atom_pair_to_var{$var});
        $dst[$atom_pair_to_var{$var}] = $src[$v];
    }
#    exit;
    return join(':', @dst);
}

################################################################################

sub permute_two{
    my ($a1, $a2, $rperm, $rnew) = @_;

    foreach my $p (@{$rperm}){

	#print "orig >> ", @{$p}, "\n";
	my $np = scalar @{$p};
	my @ni = (-1, -1);
	for(my $i = 0; $i < $np; ++$i){
	    if($p->[$i] == $a1){
		$ni[0] = $i;
	    }elsif($p->[$i] == $a2){
		$ni[1] = $i;
	    }
	}

	die "Could not find $a1 or $a2 in @{$p}\n"
            unless (($ni[0] != -1) && ($ni[1] != -1));

	my @newp = ();
	for(my $i = 0; $i < $np; ++$i){
	    push @newp, $p->[$i];
	}
	$newp[$ni[0]] = $a2;
	$newp[$ni[1]] = $a1;

	  
#	print "new  << ", @newp, "\n";

	push @{$rnew}, \@newp;

    }
}

################################################################################

sub print_one {
    my ($name, $i, $m, $r_permutations) = @_;
    my %terms;
    my @tmp_permutations = @{$r_permutations};
    for my $p (@tmp_permutations) {
        my $xx = permute_monomial($m, $p);
        my @yy = split(':', $xx);
        my @pp;
        for (my $n = 0; $n < scalar @yy; ++$n) {
            next unless $yy[$n] > 0;
            for (my $p = 0; $p < $yy[$n]; ++$p) {
#                push @pp, (sprintf "x%2.2d", $n + 1);
                push @pp, (sprintf "xO[$n]" );
            }
        }
        my $curr = join('*', sort @pp);
        $terms{$curr} = undef;
    }
    my $newname = "p".$name;
    print "    $newname\[$i] = ", join(' + ', keys %terms), ";\n";
}

################################################################################

sub print_orbits {
    my ($name, $i0, $orbs, $permutations) = @_;

    foreach (@{$orbs}) {
        print_one($name, $i0++, $_, $permutations);
    }
}

################################################################################

sub print_poly{
    my ($name, $i0, $rorbs, $r_permutations) = @_;

    my @orbs = @{$rorbs};
    foreach my $orb (@orbs){
#	print $orb, "  ", @{$orb}, "\n";
#	exit;
	&print_orbits($name, $i0, $orb, $r_permutations);
	$i0 += scalar @{$orb};
        print "\n";
    }
    print "\n\n";
}

################################################################################
