#!/usr/bin/perl

$| = 1;

use strict;
use warnings;

#
# monomial is a list of length L <= N*(N-1)/2 listing the powers of the
# variables; e.g., (1, 2, 0, ..., 0) <=> x1*x2*x2; total degree is just
# the sum of the elements
#
# permutation initially acts on the atoms and so is represented by a list
# of length N : (0, 1, 2, 3, ..., N) is "identity", and so on
#
# for (H2O)x2 the atoms are: Oa Ha1 Ha2 Ob Hb1 Hb2
# and possible permutations permute Hydrogens withing the molecules
# and molecules themselves
#

my $i0 = 0;
my $nmolec = 2;
my @atoms = qw(Oa Ha1 Ha2 La1 La2 Ob Hb1 Hb2 Lb1 Lb2);
my %atom_idx;
for (my $n = 0; $n < scalar @atoms; ++$n) {
    $atom_idx{$atoms[$n]} = $n;
}

my @permutations;
my %l_perm;

my @lmap = ("3:4","3:8","3:9","4:8","4:9","8:9");
my @hmap = ("1:2","1:6","1:7","2:6","2:7","6:7");
my @omap = ("0:5");

foreach my $h12 (0, 1) {
    foreach my $h16 (0, 1) {
    foreach my $h17 (0, 1) {
    foreach my $h26 (0, 1) {
    foreach my $h27 (0, 1) {
    foreach my $h67 (0, 1) {

my $l34 = 0;
my $l38 = 0;
my $l39 = 0;
#    foreach my $l34 (0, 1) {
#    foreach my $l38 (0, 1) {
#    foreach my $l39 (0, 1) {
    foreach my $l48 (0, 1) {
    foreach my $l49 (0, 1) {
    foreach my $l89 (0, 1) {

    foreach my $o05 (0, 1) {
	my @h_mask = ($h12, $h16, $h17, $h26, $h27, $h67);
	my @l_mask = ($l34, $l38, $l39, $l48, $l49, $l89);
	my @o_mask = ($o05);
	my $tmp = &permute_all(\@h_mask, \@hmap, 
                               \@o_mask, \@omap,
                               \@l_mask, \@lmap);
	$l_perm{join(':', @{$tmp})} = undef;
    }

    }
    }
    }
#    }
#    }
#    }

    }
    }
    }
    }
    }
}

@permutations = keys %l_perm;

print STDERR scalar @permutations, " permutations:\n";
foreach (@permutations) {
    my @toks = split(/:/,$_);
    my @arr = ();
    $_ = \@arr;
    for (my $n = 0; $n < scalar @atoms; ++$n) {
	push @arr, $atom_idx{$toks[$n]};
    }

    print STDERR "    ", join(' ', @{$_}), "\n";
}

#
# now find how the permutations act on the monomials
# composed out of (some) distances
#

my @variables = (
    # H-H
                 ['Ha1', 'Ha2'],
                 ['Ha1', 'Hb1'],
                 ['Ha1', 'Hb2'],
                 ['Ha2', 'Hb1'],
                 ['Ha2', 'Hb2'],
                 ['Hb1', 'Hb2'],
    # O-H
                 ['Oa', 'Ha1'],
                 ['Oa', 'Ha2'],
                 ['Oa', 'Hb1'],
                 ['Oa', 'Hb2'],
                 ['Ob', 'Ha1'],
                 ['Ob', 'Ha2'],
                 ['Ob', 'Hb1'],
                 ['Ob', 'Hb2'],
    # O-O
                 ['Oa', 'Ob'], 
    # L-L, excluding intramolecular H-L
                 ['La1', 'Lb1'],
                 ['La1', 'Lb2'],
                 ['La2', 'Lb1'],
                 ['La2', 'Lb2'],
    # O-L, excluding intramolecular H-L
                 ['Oa', 'Lb1'],
                 ['Oa', 'Lb2'],
                 ['Ob', 'La1'],
                 ['Ob', 'La2'],
    # H-L, excluding intramolecular H-L
                 ['Ha1', 'Lb1'],
                 ['Ha1', 'Lb2'],
                 ['Ha2', 'Lb1'],
                 ['Ha2', 'Lb2'],
                 ['Hb1', 'La1'],
                 ['Hb1', 'La2'],
                 ['Hb2', 'La1'],
                 ['Hb2', 'La2']);

print STDERR "\n";
my $nvar = scalar @variables;
my %atom_pair_to_var;
for (my $v = 0; $v < $nvar; ++$v) {
    my $key = join(':', sort @{$variables[$v]});
    print STDERR $v, " => ", $key, "\n";
    $atom_pair_to_var{$key} = $v;
}
print STDERR "\n";

#
# now loop over 1st degree monomials [we want 2B-terms only]
#

my %mono_all;
for (my $k1 = 0; $k1 < $nvar; ++$k1) {
        my @mono = (0) x $nvar;
        $mono[$k1] += 1;
        next unless is_2B(\@mono);
#        print STDERR join(':', @mono), "\n";
        $mono_all{join(':', @mono)} = undef;
}

#print STDERR "\n";
print STDERR scalar keys %mono_all, " possible 2B, 1st degree monomials\n";
my $mono1_orb = find_orbits(\%mono_all);

print STDERR scalar @{$mono1_orb}, " 1st degree terms\n";

#
# now loop over 2nd degree monomials [we want 2B-terms only]
#

for (my $k1 = 0; $k1 < $nvar; ++$k1) {
    for (my $k2 = 0; $k2 < $nvar; ++$k2) {
        my @mono = (0) x $nvar;
        $mono[$k1] += 1;
        $mono[$k2] += 1;
        next unless is_2B(\@mono);
#        print STDERR join(':', @mono), "\n";
        $mono_all{join(':', @mono)} = undef;
    }
}

#print STDERR "\n";
print STDERR scalar keys %mono_all, " possible 2B, 2nd degree monomials\n";
my $mono2_orb = find_orbits(\%mono_all);

print STDERR scalar @{$mono2_orb}, " 2nd degree terms\n";

print "\n";
#for (my $v = 0; $v < $nvar; ++$v) {
#    print "    x[$v] = morse($variables[$v]->[0], $variables[$v]->[1]);\n"
#}
#
#print "\n\n";

my @orbs = ($mono1_orb, $mono2_orb);

&print_poly("La1", $i0, \@orbs, \@permutations);

    ## permute a1 <-> a2 ##

my @new_permutations = ();
&permute_two(3, 4, \@permutations, \@new_permutations);
&print_poly("La2", $i0, \@orbs, \@new_permutations);

    ## permute a1 <-> b1 ##

@new_permutations = ();
&permute_two(3, 8, \@permutations, \@new_permutations);
&print_poly("Lb1", $i0, \@orbs, \@new_permutations);

    ## permute a1 <-> b2 ##

@new_permutations = ();
&permute_two(3, 9, \@permutations, \@new_permutations);
&print_poly("Lb2", $i0, \@orbs, \@new_permutations);

################################################################################

sub permute_all {
    my ($hmask, $hmap, $omask, $omap, $lmask, $lmap) = @_;

    my @dimer = qw(Oa Ha1 Ha2 La1 La2 Ob Hb1 Hb2 Lb1 Lb2);

    for (my $m = 0; $m < scalar @{$hmask}; ++$m) {
        if ($hmask->[$m] == 1) {
	    my @toks = split(/:/,$hmap->[$m]);
	    my $inda = $toks[0];
	    my $indb = $toks[1];

            my $tmp = $dimer[$inda];
            $dimer[$inda] = $dimer[$indb];
            $dimer[$indb] = $tmp;
        }
    }

    for (my $m = 0; $m < scalar @{$omask}; ++$m) {
        if ($omask->[$m] == 1) {
	    my @toks = split(/:/,$omap->[$m]);
	    my $inda = $toks[0];
	    my $indb = $toks[1];

            my $tmp = $dimer[$inda];
            $dimer[$inda] = $dimer[$indb];
            $dimer[$indb] = $tmp;
        }
    }

    for (my $m = 0; $m < scalar @{$lmask}; ++$m) {
        if ($lmask->[$m] == 1) {
	    my @toks = split(/:/,$lmap->[$m]);
	    my $inda = $toks[0];
	    my $indb = $toks[1];

            my $tmp = $dimer[$inda];
            $dimer[$inda] = $dimer[$indb];
            $dimer[$indb] = $tmp;
        }
    }

    return \@dimer;
}

################################################################################

sub is_2B {
    my $mono = shift;
    my %mols;

    die unless $nvar == scalar @{$mono};
    for (my $v = 0; $v < $nvar; ++$v) {
        next unless $mono->[$v] > 0;
        my $var = $variables[$v];
        my $m1 = substr ($var->[0], 1, 1);
        my $m2 = substr ($var->[1], 1, 1);
        $mols{$m1} = undef;
        $mols{$m2} = undef;
    }

    my $nmol = scalar keys %mols;

    return ($nmol == 2 ? 1 : 0);
}

################################################################################

sub find_orbits {
    my $all = shift;

    my @orbits;

#    print STDERR "\n       0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0\n";

    # cycle through the unsymmetrized monomials (0:0:1:0:2:0:0), stored in $curr
    while (1) {
        my @still_there = keys %{$all};
        last if scalar @still_there == 0;
        my $curr = shift @still_there;

        push @orbits, $curr;
#	printf STDERR "\n%-3d%s%s\n", scalar @orbits, " -> ", $curr;

	# now, given a starting unsymmetrized monomial ($curr, 0:0:1:0:2:0:0),
	# cycle through all possible permutations of atom pairs and remove
	# all redundant monomials that are consistent with those permutations
	# by deleting them from the %{$all} hash

        for my $p (@permutations) {
            my $permuted;
            my $acceptable = permute_monomial($curr, $p, \$permuted);
	    if($acceptable == 1){
#	    print " >> ", @{$p}, " | ", $permuted, "\n";
		delete ${$all}{$permuted};
	    }
        }
    }

    return \@orbits;
}

################################################################################

sub permute_monomial {
    my ($src_str, $p, $r_xx) = @_;

    my @src = split(':', $src_str);
    my $nvar = scalar @src;
    my @dst = (0) x $nvar;

    # p is array ref for array of atoms numbers, e.g., 012345
    # src_str is distance combination (variables): 0:0:0:0:0:0:1:0:1:0:0:0:0:0:0
#    print $nvar, " -- ", $src_str, "\n", @{$p}, "\n";
#    exit;

    my $acceptable = 1;

    for (my $v = 0; $v < $nvar; ++$v) {

	# cycle through the possible monomial variables and only examine those
	# with a non-zero exponentiation 
        next unless $src[$v] > 0;

        my $atms = $variables[$v];
        my ($a1, $a2) = ($atom_idx{$atms->[0]}, $atom_idx{$atms->[1]});

	# $a1 and $a2 are atom numbers, for Oa-Ob could be 0,5
	# if the two atoms cannot be permuted, then
	#     p would equal (0,x,x,x,x,5,x,x,x,x) and @permuted = ("Oa","Ob")
	# if the two atoms can    be permuted, then
	#     p would equal (5,x,x,x,x,0,x,x,x,x) and @permuted = ("Ob","Oa")
	# in the above x is arbitrary (permuation independent of its value)

        my @permuted = ($atoms[$p->[$a1]], $atoms[$p->[$a2]]);
        my $var = join(':', sort @permuted);

#	print STDERR "$v : ", $src[$v], "  ", $atms->[0], "  $a1  ", 
#	                                      $atms->[1], "  $a2  \n";
#	print STDERR "permute : ", $p->[$a1], " <-> ", $p->[$a2], "\n";
#	print STDERR @permuted, "\n";
#	print STDERR $var, "\n";

	if(exists($atom_pair_to_var{$var})){
    	    $dst[$atom_pair_to_var{$var}] = $src[$v];
	}else{
	    $acceptable = 0;
	}
    }

    $$r_xx = join(':', @dst);
    return $acceptable;
}

################################################################################

sub permute_two{
    my ($a1, $a2, $rperm, $rnew) = @_;

    foreach my $p (@{$rperm}){

	my $np = scalar @{$p};
	my @ni = (-1, -1);
	for(my $i = 0; $i < $np; ++$i){
	    if($p->[$i] == $a1){
		$ni[0] = $i;
	    }elsif($p->[$i] == $a2){
		$ni[1] = $i;
	    }
	}

	die "Could not find $a1 or $a2 in @{$p}\n"
            unless (($ni[0] != -1) && ($ni[1] != -1));

	my @newp = ();
	for(my $i = 0; $i < $np; ++$i){
	    push @newp, $p->[$i];
	}
	$newp[$ni[0]] = $a2;
	$newp[$ni[1]] = $a1;

	  
	push @{$rnew}, \@newp;

    }
}

################################################################################

sub print_one {
    my ($name, $i, $m, $r_permutations) = @_;

    my %terms;
    my @tmp_permutations = @{$r_permutations};

    for my $p (@tmp_permutations) {
	my $xx;
        my $acceptable = permute_monomial($m, $p, \$xx);
        if($acceptable == 1){

	    # @yy contains the powers of each variable of the permuted monomial
            my @yy = split(':', $xx);
            my @pp = ();
            for (my $n = 0; $n < scalar @yy; ++$n) {
                next unless $yy[$n] > 0;
	        # for each non-zero exponentiated variable, record the 
	        # variable products in @pp
                for (my $p = 0; $p < $yy[$n]; ++$p) {
#                    push @pp, (sprintf "x%2.2d", $n + 1);
                    push @pp, (sprintf "xL[$n]" );
                }
            }
    	    my $curr = join('*', sort @pp);
	    $terms{$curr} = undef;
	}
    }

    my $newname = "p".$name;
    print "    $newname\[$i] = ", join(' + ', keys %terms), ";\n";

}

################################################################################

sub print_orbits {
    my ($name, $i0, $orbs, $permutations) = @_;

    foreach (@{$orbs}) {
	# $_ contains the unique monomial, e.g., 0:0:1:0:2:0:0
        print_one($name, $i0++, $_, $permutations);
    }

}

################################################################################

sub print_poly{
    my ($name, $i0, $rorbs, $r_permutations) = @_;

    # here, orbs is the array containing the 1st-order orbits, 2nd-order, 3rd...
    # below, go through and print each order at a time

    my @orbs = @{$rorbs};
    foreach my $orb (@orbs){
	&print_orbits($name, $i0, $orb, $r_permutations);
	$i0 += scalar @{$orb};
        print "\n";
    }
    print "\n\n";

}

################################################################################
