#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cassert>
#include <cstdlib>

#include <iomanip>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#include <cmath>
#include <algorithm>

#include "read-hist.h"
#include "read-bin.h"
#include "pbc-tools.h"

#define SHIFT_HALF_CELL yes

////////////////////////////////////////////////////////////////////////////////

namespace {

////////////////////////////////////////////////////////////////////////////////

const double dz = 0.01;

double z_length = 0;
double half_z_length = 0;

int nbins = 0;
int natm = 0;
double* bins;

bool first(true);

//----------------------------------------------------------------------------//

size_t do_frames(size_t lineno, std::ifstream& ifs)
{

    size_t frameno(0);

    while(!ifs.eof()){

	opt::hist_frame frame;

#ifdef HIST     
	opt::read_hist_frame(lineno, ifs, frame);
#else
	opt::read_cmd_frame(lineno, ifs, frame);
#endif

	if(frame.natm > 0){
	    ++frameno;

#ifdef SHIFT_HALF_CELL
	    tools::center_slab_at_origin(frame.natm, frame.cell, frame.mass,
					 frame.xyz, true);
#else
	    tools::center_slab_at_origin(frame.natm, frame.cell, frame.mass,
					 frame.xyz, false);
#endif

    	    // Set up bins for storing density profile

    	    if(first){
		first = false;

    		z_length = frame.cell[8]; 
		half_z_length = z_length/2.0;

		natm = frame.natm;

    		nbins = (int)(z_length/dz);
		bins = new double[(int)(z_length/dz)];

    		std::fill(bins, bins + nbins, 0.0);
	    }


    	    for(size_t i = 0; i < frame.natm; i += 3){
    		//const int binid = (int)(frame.xyz[3*i + 2]/dz);
	    	++bins[(int)((frame.xyz[3*i + 2]+half_z_length)/dz)];
	    }

	}
    }
    return frameno;

}

////////////////////////////////////////////////////////////////////////////////

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{
    if (argc < 2) {
        std::cerr << "usage: density_profile POSITION_CMD > profile.dat" 
	          << std::endl;
        return EXIT_FAILURE;
    }

    size_t tot_frameno(0);

    for(int i = 1; i < argc; ++i){

	std::ifstream ifs(argv[i]);
    	std::string line;
    	size_t lineno(0);
#ifdef HIST
	std::getline(ifs, line);
    	std::getline(ifs, line);
    	lineno += 2;
#endif

	const size_t frameno = do_frames(lineno, ifs);
	tot_frameno += frameno;

	std::cerr << argv[i] << " : " << frameno << std::endl;

    }


    std::cout << std::setprecision(10) << std::scientific;

    // changed size_t to int
    for(int i = 0; i < nbins; ++i){
	const double z = dz*(i + 0.5);

	std::cout << std::setw(25) << z - half_z_length
	          << std::setw(25) << bins[i]/(natm/3.0)/tot_frameno
		  << std::endl;

    }

    delete[] bins;

}

