#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cassert>
#include <cstdlib>

#include <iomanip>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#include <cmath>
#include <algorithm>

#include "read-hist.h"
#include "read-bin.h"
#include "pbc-tools.h"

//#define SHIFT_HALF_CELL yes

#define HIST yes

////////////////////////////////////////////////////////////////////////////////

namespace {

////////////////////////////////////////////////////////////////////////////////

const double cg_length = 2.4;
const double cg_cutoff = cg_length*3;

const double dx = 1.00;
const double dy = 1.00;
const double dz = 1.00;

double z_length = 0;
double half_z_length = 0;

double avg(0);
int num_avg(0);

const size_t skip(100);

int ngrid_x = 0;
int ngrid_y = 0;
int ngrid_z = 0;

int natm = 0;
int nbins = 0;
double* bins;
int num_bins = 0;

bool first(true);

//----------------------------------------------------------------------------//

double cg_density(const double r, const double xi)
{
    const double xi2 = xi*xi;

    return std::pow(2*M_PI*xi2, -3/2)*std::exp(-r*r/(2*xi2));
}

//----------------------------------------------------------------------------//

size_t do_frames(size_t lineno, std::ifstream& ifs)
{

    size_t frameno(0);

    while(!ifs.eof()){

	opt::hist_frame frame;

#ifdef HIST     
	opt::read_hist_frame(lineno, ifs, frame);
#else
	opt::read_cmd_frame(lineno, ifs, frame);
#endif

	if(frame.natm > 0){
	    ++frameno;

#ifdef SHIFT_HALF_CELL
	    tools::center_slab_at_origin(frame.natm, frame.cell, frame.mass,
					 frame.xyz, true);
#else
	    tools::center_slab_at_origin(frame.natm, frame.cell, frame.mass,
					 frame.xyz, false);
#endif

	    if(frameno % skip == 0){

	    // set up grid: dim_1 = cell_x / dx
	    //              dim_2 = cell_y / dy
	    //              dim_3 = cell_z / dz

    	    if(first){
		first = false;

    		z_length = frame.cell[8]; 
		half_z_length = z_length/2.0;

		natm = frame.natm;

    		ngrid_x = (int)(frame.cell[0]/dx);
    		ngrid_y = (int)(frame.cell[4]/dy);
    		ngrid_z = (int)(frame.cell[8]/dz);

		nbins = ngrid_z/2;
		bins = new double[nbins];
		std::cerr << "allocated nbins = " << nbins << std::endl;
	    }

	    for(int ix = 0; ix < ngrid_x; ++ix){
		const double x = (ix + 0.5)*dx;

		for(int iy = 0; iy < ngrid_y; ++iy){
		    const double y = (iy + 0.5)*dy;

		    for(int iz = 0; iz < ngrid_z; ++iz){
		      const double z = (iz + 0.5)*dz - half_z_length;
		      const double zabs = std::fabs(z);

		      if(zabs < 20.0){
			double grid_point[3] = {x, y, z};
			double value(0);
			for(size_t j = 0; j < frame.natm; j += 3){ // O only
			    const double* qj = frame.xyz + 3*j;
			    const double rij = tools::distance3(frame.imcon,
		    						frame.cell,
								frame.rcell,
								grid_point, qj);
			    if(rij < cg_cutoff)
				value += cg_density(rij, cg_length);
			}

			bins[(int)(zabs/dz)] += value;
//			std::cerr << z << ' ' << value << std::endl;
		        if(zabs < 7.5){
		          avg += value;
		          ++num_avg;
		        }
		      }
		    }
		    num_bins += 2;
		}
	    }

	    }

	}
    }
    return frameno;

}

////////////////////////////////////////////////////////////////////////////////

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{
    if (argc < 2) {
        std::cerr << "usage: density_profile POSITION_CMD > profile.dat" 
	          << std::endl;
        return EXIT_FAILURE;
    }

    size_t tot_frameno(0);

    std::cout << std::setprecision(10) << std::scientific;

    for(int i = 1; i < argc; ++i){

	std::ifstream ifs(argv[i]);
    	std::string line;
    	size_t lineno(0);
#ifdef HIST
	std::getline(ifs, line);
    	std::getline(ifs, line);
    	lineno += 2;
#endif

	const size_t frameno = do_frames(lineno, ifs);
	tot_frameno += frameno;

	std::cerr << argv[i] << " : " << frameno << std::endl;

//	std::cerr << "average density: " << avg/num << std::endl
//	          << "averaged over " << tot_frameno << " frames" << std::endl;

    }

    std::cerr << "average density: " << avg/num_avg << std::endl
	      << "averaged over " << tot_frameno << " frames" << std::endl;


#if 1
    // changed size_t to int
    for(int i = 0; i < nbins; ++i){
	const double z = dz*(i + 0.5);

	std::cout << std::setw(25) << z
//	          << std::setw(25) << bins[i]/(natm/3.0)/num_bins
	          << std::setw(25) << bins[i]/num_bins
		  << std::endl;

    }

    delete[] bins;
#endif

}
