#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>
#include <cstdlib>

#include <iomanip>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#include "read-hist.h"
#include "read-xyz.h"
#include "constants.h"

////////////////////////////////////////////////////////////////////////////////

namespace {

////////////////////////////////////////////////////////////////////////////////

static std::vector<opt::hist_frame> frames;

////////////////////////////////////////////////////////////////////////////////

} // namespace

////////////////////////////////////////////////////////////////////////////////
double SHELL_RADIUS=0.0;
double BOX_WIDTH=0.0;

int main(int argc, char** argv)
{
    if (argc != 3) {
       std::cerr << "usage: bulk_dipole SHELL_RADIUS BOX_WIDTH > STDOUT" << std::endl;
        return EXIT_FAILURE;
    }

    std::istringstream iss(argv[1]);
    iss >> SHELL_RADIUS;

    std::istringstream is(argv[2]);
    is >> BOX_WIDTH;

    std::cerr << SHELL_RADIUS << "\t" << SHELL_RADIUS / 2.0 << std::endl;
    std::cerr << BOX_WIDTH << "\t" << BOX_WIDTH / 2.0 << std::endl;

    try {

	std::vector<opt::hist_frame> this_file;
    	opt::read_cmd(this_file, true);
    	std::copy(this_file.begin(), this_file.end(),
    		  std::back_inserter(frames));

	std::cerr <<" CMD : " << this_file.size() << std::endl;

    } catch (const std::exception& e) {
	std::cerr << e.what() << std::endl;
	return EXIT_FAILURE;
    }

// write cluster positions

    for(size_t i = 0; i < frames[0].natm; i+=3){

	std::ofstream position_file;
	std::string base = "pos_1B+NB_cluster_";
	std::string tail = ".dat";

	std::ostringstream oss;
	oss << (i/3);
	std::string file_num;
	file_num = oss.str();
	
	//cycle through the trajectory to print out dynamic cluster information

	for(size_t k = 0; k < 1 /*frames.size()*/; ++k){

	    std::ostringstream oso;
	    oso << k;
	    std::string frame_num;
	    frame_num = oso.str();

	    std::string filename = base + file_num + "_frame_" + frame_num + tail;

	    position_file.open(filename.c_str());

	    //need an ordered array of water molecules by distance from cluster center

	    double x_distances[frames[k].natm];
	    std::fill(x_distances, x_distances + frames[k].natm, 0.0);

	    double y_distances[frames[k].natm];
	    std::fill(y_distances, y_distances + frames[k].natm, 0.0);

	    double z_distances[frames[k].natm];
	    std::fill(z_distances, z_distances + frames[k].natm, 0.0);

	    double distances[frames[k].natm];
	    std::fill(distances, distances + frames[k].natm, 0.0);
	    
	    for(size_t m = 0; m < frames[k].natm; ++m){

		size_t m3 = 3*m; //index of atom wrt cluster center
		size_t i3 = 3*i; //index of cluster center oxygen

		double d_xyz[3];

		for(size_t o = 0; o < 3; ++o){

		    d_xyz[o] = frames[k].xyz[m3+o] - frames[k].xyz[i3+o];

		    if (d_xyz[o] >=   BOX_WIDTH/2.0) d_xyz[o] -= BOX_WIDTH;		
		    if (d_xyz[o] < -1*BOX_WIDTH/2.0) d_xyz[o] += BOX_WIDTH;		

		}


		double dist_from_cluster_center = std::sqrt((d_xyz[0] * d_xyz[0])
				+ (d_xyz[1] * d_xyz[1]) + (d_xyz[2] * d_xyz[2]));

		distances[m] = dist_from_cluster_center;
		
		x_distances[m] = d_xyz[0];
		y_distances[m] = d_xyz[1];
		z_distances[m] = d_xyz[2];

	    }

	    for(size_t p = 0; p < frames[0].natm; p+=3){

		if (distances[p] > SHELL_RADIUS) continue;

	        position_file << "O\t"  
                              << x_distances[p + 0]  << "\t" 
                              << y_distances[p + 0]  << "\t" 
                              << z_distances[p + 0]  << "\t"
			      << distances[p + 0]    << std::endl;

	        position_file << "H\t"  
                              << x_distances[p + 1]  << "\t" 
                              << y_distances[p + 1]  << "\t" 
                              << z_distances[p + 1]  << "\t"
			      << distances[p + 1]      << std::endl;

	        position_file << "H\t"  
                              << x_distances[p + 2]  << "\t" 
                              << y_distances[p + 2]  << "\t" 
                              << z_distances[p + 2]  << "\t"
			      << distances[p + 2]      << std::endl;
	    }
	
	    position_file.close();
        }
    }
}

