#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>
#include <cstdlib>

#include <iomanip>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#include "read-hist.h"
#include "read-xyz.h"
#include "constants.h"

////////////////////////////////////////////////////////////////////////////////

namespace {

////////////////////////////////////////////////////////////////////////////////

static std::vector<opt::hist_frame> frames;

////////////////////////////////////////////////////////////////////////////////

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{
    if (argc != 1) {
        std::cerr << "usage: bulk_dipole ..." << std::endl;
        return EXIT_FAILURE;
    }

    try {

	std::vector<opt::hist_frame> this_file;
    	opt::read_cmd(this_file, true);
    	std::copy(this_file.begin(), this_file.end(),
    		  std::back_inserter(frames));

	std::cerr <<" CMD : " << this_file.size() << std::endl;

    } catch (const std::exception& e) {
	std::cerr << e.what() << std::endl;
	return EXIT_FAILURE;
    }

//    Open files for Writing

    std::cout.setf(std::ios_base::showpoint);
    std::cout.precision(9);

    for(size_t n = 0; n < frames.size(); ++n){

	const size_t nmolec = frames[n].natm/3;

	double orig_dip[3];
	std::fill(orig_dip, orig_dip + 3, 0.0);

	for(size_t i = 0; i < nmolec; ++i){
	    size_t i3 = 3*i;
	    for(size_t j = 0; j < 3; ++j){
		orig_dip[j]  += frames[n].dipmol[i3 + j]; 
//		                + frames[n].dipind[i3 + j];
	    }
	}

	const double orig_dip_mag = std::sqrt(orig_dip[0]*orig_dip[0]
				            + orig_dip[1]*orig_dip[1]
					    + orig_dip[2]*orig_dip[2]);

	std::cout << frames[n].time << ' ' 
	          << orig_dip[0]    << ' ' 
	          << orig_dip[1]    << ' ' 
		  << orig_dip[2]    << ' '
		  << orig_dip_mag   << std::endl;

    }

}

