#include <cmath>
#include <vector>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <algorithm>

namespace {

static size_t ntraj = 0;
static int nsamples = 0;

static double t0;
static double dt;

static std::vector<double> accum;
static std::vector<double> ntrajs;

const double debye = 3.33564e-30; // Coulomb*m

void do_autocorr(const char* filename, const int traj_num)
{
    std::vector<double> v3x;
    std::vector<double> v3y;
    std::vector<double> v3z;

    std::ifstream ifs(filename);

    if (!ifs) {
        std::cerr << " ** Error ** : could not open '"
                  << filename << "' for reading"
                  << std::endl;
        exit(1);
    }

//    std::string line;
//    getline(ifs,line);
//    std::istringstream iss(line);
//    iss >> vol;

    while (ifs) {
        double t, dx, dy, dz, dm;
        ifs >> t >> dx >> dy >> dz >> dm;

        if (v3x.size() == 0 && ntraj == 0)
            t0 = t;

        if (v3x.size() == 1 && ntraj == 0)
            dt = t - t0;

        if (ifs) {
            v3x.push_back(dx*debye);
            v3y.push_back(dy*debye);
            v3z.push_back(dz*debye);
        }
    }


    int this_file_nsamples = (int) v3x.size();

    double corr_x[this_file_nsamples];
    double corr_y[this_file_nsamples];
    double corr_z[this_file_nsamples];
    double ntime[this_file_nsamples];

    std::fill(corr_x, corr_x + this_file_nsamples, 0.0);
    std::fill(corr_y, corr_y + this_file_nsamples, 0.0);
    std::fill(corr_z, corr_z + this_file_nsamples, 0.0);
    std::fill(ntime, ntime + this_file_nsamples, 0.0);

    for(size_t i = 0; i < this_file_nsamples; ++i){
	for(size_t j = i; j < this_file_nsamples; ++j){
	    size_t time = (j-i);

	    corr_x[time] += v3x[i]*v3x[j];
	    corr_y[time] += v3y[i]*v3y[j];
	    corr_z[time] += v3z[i]*v3z[j];
	    ++ntime[time];
	}
    }

    for(size_t i = 0; i < this_file_nsamples; ++i){
	corr_x[i] /= ntime[i];
    	corr_y[i] /= ntime[i];
    	corr_z[i] /= ntime[i];
    }

    std::cerr << "* loaded " << this_file_nsamples
              << " samples from '" << filename << "'"
              << std::endl;

    if (nsamples < this_file_nsamples){

	for (int k = 0; k < nsamples; ++k) {
	    const double autocorr =
		(corr_x[k] + corr_y[k] + corr_z[k]);

	    accum[k] += autocorr;
	    ++ntrajs[k];
	}

	for (int k = nsamples; k < this_file_nsamples; ++k) {
	    const double autocorr =
		(corr_x[k] + corr_y[k] + corr_z[k]);

	    accum.push_back(autocorr);
	    ntrajs.push_back(1.0);
	}

	nsamples = this_file_nsamples;

    }else{
	for (int k = 0; k < this_file_nsamples; ++k) {
	    const double autocorr =
		(corr_x[k] + corr_y[k] + corr_z[k]);
	    
	    accum[k] += autocorr;
	    ++ntrajs[k];
	}

    }

    ++ntraj;
}

} // namespace

int main(int argc, char **argv)
{
    if (argc < 2){
	std::cerr << "Usage : ./response_ir dipole_surface_1 .... > "
		  << "averaged_dipole_autocorr" << std::endl;
    }

    for (int i = 1; i < argc; ++i)
        do_autocorr(argv[i], i);

    if (ntraj == 0)
        exit(0);

    std::cout << std::setprecision(8) << std::scientific;

    for (int k = 0; k < nsamples; ++k) {
        std::cout << std::setw(18) << dt*k
                  << std::setw(18) << (accum[k]/ntrajs[k])
                  << '\n';
    }
}
