#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>
#include <cstdlib>

#include <iomanip>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#include "read-hist.h"
#include "read-xyz.h"
#include "constants.h"

////////////////////////////////////////////////////////////////////////////////

namespace {

////////////////////////////////////////////////////////////////////////////////

static std::vector<opt::hist_frame> frames;

////////////////////////////////////////////////////////////////////////////////

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{
    if (argc != 3) {
       std::cerr << "usage: molec_dip_1bNb > STDOUT" << std::endl;
        return EXIT_FAILURE;
    }

    try {

	std::vector<opt::hist_frame> this_file;
    	opt::read_cmd(this_file, true);
    	std::copy(this_file.begin(), this_file.end(),
    		  std::back_inserter(frames));

	std::cerr <<" CMD : " << this_file.size() << std::endl;

    } catch (const std::exception& e) {
	std::cerr << e.what() << std::endl;
	return EXIT_FAILURE;
    }

//    Open files for Writing

    std::cout.setf(std::ios_base::showpoint);
    std::cout.precision(9);

    for(size_t n = 0; n < frames.size(); ++n){

	const size_t nmolec = frames[n].natm/3;

	double orig_dip[3];
	std::fill(orig_dip, orig_dip + 3, 0.0);

	for(size_t i = 0; i < nmolec; ++i){
	    size_t i3 = 3*i;
	    for(size_t j = 0; j < 3; ++j){
		orig_dip[j]  += frames[n].dipmol[i3 + j] 
  	                      + frames[n].dipind[i3 + j];
	    }
	}

	const double orig_dip_mag = std::sqrt(orig_dip[0]*orig_dip[0]
				            + orig_dip[1]*orig_dip[1]
					    + orig_dip[2]*orig_dip[2]);

	std::cout << frames[n].time << ' ' 
	          << orig_dip[0]    << ' ' 
	          << orig_dip[1]    << ' ' 
		  << orig_dip[2]    << ' '
		  << orig_dip_mag   << std::endl;

    }

// write molecule zero dipole (specific for HOD-h2o sims)    

    for(size_t n = 0; n < (frames[0].natm/3); ++n){

        std::ofstream molecular_file;

        std::string base = "dip1b_1B_mol_";
        std::string tail = ".dat";

	std::ostringstream oss;
	oss << n;
	std::string file_num;
	file_num = oss.str();

	std::string filename = base + file_num + tail;
	molecular_file.open(filename.c_str());

	for(size_t k = 0; k < frames.size(); ++k){

            //const size_t nmolec = frames[k].natm/3;

            double orig_dip[3];
            std::fill(orig_dip, orig_dip + 3, 0.0);

            for(size_t i = n; i < (n+1); ++i){
                size_t i3 = 3*i;
                for(size_t j = 0; j < 3; ++j){
                    orig_dip[j]  += frames[k].dipmol[i3 + j]
                                    + frames[k].dipind[i3 + j];
                }
            }

            const double orig_dip_mag = std::sqrt(orig_dip[0]*orig_dip[0]
                                                + orig_dip[1]*orig_dip[1]
                                                + orig_dip[2]*orig_dip[2]);

            molecular_file << frames[k].time << ' '
                           << orig_dip[0]    << ' '
                           << orig_dip[1]    << ' '
                           << orig_dip[2]    << ' '
                           << orig_dip_mag   << std::endl;

	}

 

        molecular_file.close();

    }
}

