#include <cmath>
#include <vector>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <algorithm>

namespace {

static size_t ntraj = 0;
static int nsamples = 0;

static double t0;
static double dt;

static std::vector<double> accum;
static std::vector<double> ntrajs;

const double debye = 3.33564e-30; // Coulomb*m

void do_autocorr(const char* filename, const int traj_num)
{
    std::vector<double> aiso;

    std::ifstream ifs(filename);

    if (!ifs) {
        std::cerr << " ** Error ** : could not open '"
                  << filename << "' for reading"
                  << std::endl;
        exit(1);
    }

//    std::string line;
//    getline(ifs,line);
//    std::istringstream iss(line);
//    iss >> vol;

    while (ifs) {
        double t, axx, ayy, azz, dm, tmp;
        ifs >> t >> axx >> tmp >> tmp >> ayy >> tmp >> azz;

        if (aiso.size() == 0 && ntraj == 0)
            t0 = t;

        if (aiso.size() == 1 && ntraj == 0)
            dt = t - t0;

        if (ifs) {
//	    aiso.push_back(1.0/3.0*(axx)*1e-30);
//	    aiso.push_back(1.0/3.0*(axx+ayy+azz)*1e-30); // A^3 -> m^3
	    aiso.push_back(1.0/3.0*(axx+ayy+azz)); // A^3 -> m^3
        }
    }


    int this_file_nsamples = (int) aiso.size();

    double corr_iso[this_file_nsamples];
    double ntime[this_file_nsamples];

    std::fill(corr_iso, corr_iso + this_file_nsamples, 0.0);
    std::fill(ntime, ntime + this_file_nsamples, 0.0);

    for(size_t i = 0; i < this_file_nsamples; ++i){
	for(size_t j = i; j < this_file_nsamples; ++j){
	    size_t time = (j-i);

	    corr_iso[time] += aiso[i]*aiso[j];
	    ++ntime[time];
	}
    }

    for(size_t i = 0; i < this_file_nsamples; ++i)
	corr_iso[i] /= ntime[i];

    std::cerr << "* loaded " << this_file_nsamples
              << " samples from '" << filename << "'"
              << std::endl;

    if (nsamples < this_file_nsamples){

	for (int k = 0; k < nsamples; ++k) {

	    accum[k] += corr_iso[k];
	    ++ntrajs[k];
	}

	for (int k = nsamples; k < this_file_nsamples; ++k) {
	    accum.push_back(corr_iso[k]);
	    ntrajs.push_back(1.0);
	}

	nsamples = this_file_nsamples;

    }else{
	for (int k = 0; k < this_file_nsamples; ++k) {
	    accum[k] += corr_iso[k];
	    ++ntrajs[k];
	}

    }

    ++ntraj;
}

} // namespace

int main(int argc, char **argv)
{
    if(argc < 2){
	std::cerr << "usage: ./response_isotropic_raman pol.dat pol.dat > corr_iso_pol.dat" << std::endl;
	exit(1);
    }
    for (int i = 1; i < argc; ++i)
        do_autocorr(argv[i], i);

    if (ntraj == 0)
        exit(0);

    std::cout << std::setprecision(8) << std::scientific;

    for (int k = 0; k < nsamples; ++k) {
        std::cout << std::setw(18) << dt*k
                  << std::setw(18) << (accum[k]/ntrajs[k])
                  << '\n';
    }
}
