#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>
#include <cstdlib>

#include <iomanip>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cstring>

#include "read-hist.h"
#include "io-tools.h"

#define HIST yes

////////////////////////////////////////////////////////////////////////////////

namespace {

////////////////////////////////////////////////////////////////////////////////

const double dr = 0.01;
const double r_max = 9.0;

const int nbins = (int)(r_max/dr);
double bins[nbins];
//double bins[(int)(r_max/dr)];
//double bins[nbins];

double invvolm(0);
size_t pairs_per_frame(0);

//----------------------------------------------------------------------------//

//----------------------------------------------------------------------------//

inline double distance3(size_t imcon, const double* cell, const double* rcell,
		       	const double* r1, const double* r2)
{
    double dx = r1[0] - r2[0];
    double dy = r1[1] - r2[1];
    double dz = r1[2] - r2[2];

    const double ssx = rcell[0]*dx + rcell[3]*dy + rcell[6]*dz;
    const double ssy = rcell[1]*dx + rcell[4]*dy + rcell[7]*dz;
    const double ssz = rcell[2]*dx + rcell[5]*dy + rcell[8]*dz;

    const double xss = ssx - std::floor(ssx + 0.5);
    const double yss = ssy - std::floor(ssy + 0.5);
    const double zss = ssz - std::floor(ssz + 0.5);

    dx = cell[0]*xss + cell[3]*yss + cell[6]*zss;
    dy = cell[1]*xss + cell[4]*yss + cell[7]*zss;
    dz = cell[2]*xss + cell[5]*yss + cell[8]*zss;

    return std::sqrt(dx*dx + dy*dy + dz*dz);
}

//----------------------------------------------------------------------------//

size_t do_rdf(size_t lineno, std::ifstream& ifs,
  	      const char* c_atm1, const char* c_atm2)
{

    std::string atm1(c_atm1);
    std::string atm2(c_atm2);

    // Open file

//    static std::vector<opt::hist_frame> frames;
//    frames.clear();
//
//    try {
//
//    	opt::read_cmd(frames, false);
//#endif	
////    	std::copy(frames.begin(), frames.end(),
////    		  std::back_inserter(frames));
//
//	std::cerr << filename << " : " << frames.size() << std::endl;
//
//    } catch (const std::exception& e) {
//	std::cerr << e.what() << std::endl;
//	exit(1);
//    }

    size_t frameno(0);

    while(!ifs.eof()){
//    for(size_t n = 0; n < frames.size(); ++n){

	opt::hist_frame frame;

#ifdef HIST	
	opt::read_hist_frame(lineno, ifs, frame);
#else
	opt::read_cmd_frame(lineno, ifs, frame);
#endif

	size_t pairs_this_frame(0);

//	if(frame.natm < 1){
//	    std::cerr << "Too few atoms in this frame: " << frame.natm
//		      << std::endl;
//	    std::cerr << "at line number " << lineno << std::endl;
//	    exit(1);
//	}

	if(frame.natm > 0){
	    ++frameno;
	    for(size_t i = 0; i < frame.natm; ++i){

    		std::string pair_name;
		bool match1(false);
		if(opt::match_name(frame.name + i*MAX_CHAR_LENGTH, 
		    		   c_atm1)){
		    match1 = true;
		    pair_name = atm2;
		}else if(opt::match_name(frame.name + i*MAX_CHAR_LENGTH, 
		    			 c_atm2)){
		    match1 = true;
		    pair_name = atm1;
		}

    		if(match1){
		    for(size_t j = i + 1; j < frame.natm; ++j){

    			if(opt::match_name(frame.name + j*MAX_CHAR_LENGTH, 
			    		   pair_name.c_str())){

    			    const double* qi = frame.xyz + 3*i;
			    const double* qj = frame.xyz + 3*j;

    			    const double rij = distance3(frame.imcon, 
			    				 frame.cell, 
							 frame.rcell,
							 qi, qj);


    			    if(rij < r_max)
				++bins[(int)(rij/dr)];

    			    ++pairs_this_frame;
			}
		    }
		}
	    }

    	    if(pairs_per_frame == 0)
		pairs_per_frame = pairs_this_frame;
	    else if(pairs_this_frame != pairs_per_frame){
		std::cerr << "Dying. Not a constant number of atoms per frame"
		    << std::endl;
		exit(1);
	    }

    	    invvolm += 1.0/(frame.vol);
	}
    }

    return frameno;

}

////////////////////////////////////////////////////////////////////////////////

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{
    if (argc < 3) {
        std::cerr << "usage: rdf atm1 atm2 file file ... " << std::endl;
        return EXIT_FAILURE;
    }

    std::fill(bins, bins + nbins, 0.0);

    const char* c_atm1 = argv[1];
    const char* c_atm2 = argv[2];

    for(int i = 3; i < argc; ++i){

	std::ifstream ifs(argv[i]);
	std::string line;
	size_t lineno(0);
#ifdef HIST
	std::getline(ifs, line);
	std::getline(ifs, line);
	lineno += 2;
#endif

	const size_t frameno = do_rdf(lineno, ifs, c_atm1, c_atm2);

	std::cerr << argv[i] << " : " << frameno << std::endl;

    }

    // Report 

    const double norm_fac = 1.0/invvolm/pairs_per_frame;

    std::cout.precision(8);
    std::cout.setf(std::ios::fixed, std::ios::floatfield);

    for(size_t i = 0; i < nbins; ++i){
	const double r = dr*(i+0.5);

	const double rm = r - dr/2;
	const double rp = r + dr/2;

	const double dvol = 4.0/3.0*M_PI*(std::pow(rp, 3) - std::pow(rm, 3));

	std::cout << std::setw(16) << r 
	          << std::setw(16) << norm_fac*bins[i]/dvol
	          << std::endl;

    }

}
