#include <cmath>
#include <vector>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <algorithm>

#define AYY DOIT

namespace {

static size_t ntraj = 0;
static int nsamples = 0;

static double t0;
static double dt;

static std::vector<double> accum;
static std::vector<double> ntrajs;

const double debye = 3.33564e-30; // Coulomb*m

void do_autocorr(const char* filename, const int traj_num)
{
    std::vector<double> dz;
    std::vector<double> axx;
    std::vector<double> ayy;

    std::ifstream ifs(filename);

    if (!ifs) {
        std::cerr << " ** Error ** : could not open '"
                  << filename << "' for reading"
                  << std::endl;
        exit(1);
    }


    while (!ifs.eof()) {

    	std::string line;
	getline(ifs,line);
	if(ifs.eof())
	    break;
	    
	std::istringstream iss(line);
	double t, my_dz, my_axx, my_ayy;
	iss >> t >> my_dz >> my_axx >> my_ayy;

	if(iss.fail()){
	    std::cerr << " Could not parse this line into 4 doubles:"
		      << std::endl << line << std::endl
	              << "skipping file: " << filename << std::endl;
	    return;
	}

        if (axx.size() == 0 && ntraj == 0)
            t0 = t;

        if (axx.size() == 1 && ntraj == 0)
            dt = t - t0;

        if (ifs) {
	    dz.push_back(my_dz); // debye
	    axx.push_back(my_axx); // A^3
	    ayy.push_back(my_ayy); // A^3
        }
    }


    int this_file_nsamples = (int) axx.size();

    double corr[this_file_nsamples];
    double ntime[this_file_nsamples];

    std::fill(corr, corr + this_file_nsamples, 0.0);
    std::fill(ntime, ntime + this_file_nsamples, 0.0);

    for(size_t i = 0; i < this_file_nsamples; ++i){
	for(size_t j = i; j < this_file_nsamples; ++j){
	    size_t time = (j-i);

#ifdef AYY
	    corr[time] += dz[i]*(axx[j] + ayy[j])*0.5;
#else
	    corr[time] += dz[i]*axx[j];
#endif
	    ++ntime[time];
	}
    }

    for(size_t i = 0; i < this_file_nsamples; ++i)
	corr[i] /= ntime[i];

    std::cerr << "* loaded " << this_file_nsamples
              << " samples from '" << filename << "'"
              << std::endl;

    if (nsamples < this_file_nsamples){

	for (int k = 0; k < nsamples; ++k) {

	    accum[k] += corr[k];
	    ++ntrajs[k];
	}

	for (int k = nsamples; k < this_file_nsamples; ++k) {
	    accum.push_back(corr[k]);
	    ntrajs.push_back(1.0);
	}

	nsamples = this_file_nsamples;

    }else{
	for (int k = 0; k < this_file_nsamples; ++k) {
	    accum[k] += corr[k];
	    ++ntrajs[k];
	}

    }

    ++ntraj;
}

} // namespace

int main(int argc, char **argv)
{
    if(argc < 2){
	std::cerr << "usage: ./response_sfg_fcf di-pol.dat di-pol.dat > corr_sfg_di-pol.dat" << std::endl;
	exit(1);
    }
    for (int i = 1; i < argc; ++i)
        do_autocorr(argv[i], i);

    if (ntraj == 0)
        exit(0);

    std::cout << std::setprecision(8) << std::scientific;

    for (int k = 0; k < nsamples; ++k) {
        std::cout << std::setw(18) << dt*k
                  << std::setw(18) << (accum[k]/ntrajs[k])
                  << '\n';
    }
}
