#ifndef STRUCT_RAMAN_TCC_H
#define STRUCT_RAMAN_TCC_H

#include <vector>
#include <string>
#include <iostream>

namespace raman {

struct eprop {
    inline eprop();
    inline eprop(const eprop&);
    inline ~eprop();

    size_t nmolec;
    double* pol_iso;  // nmolec
    double* pol_aniso;  // nmolec
};

inline eprop::eprop()
: nmolec(0), pol_iso(0), pol_aniso(0)
{
}

inline eprop::eprop(const eprop& o)
{
    nmolec  = o.nmolec;
    if (nmolec > 0) {
	pol_iso = new double[nmolec];
    	std::copy(o.pol_iso, o.pol_iso + nmolec, pol_iso);
	pol_aniso = new double[nmolec];
    	std::copy(o.pol_aniso, o.pol_aniso + nmolec, pol_aniso);
    }
}

inline eprop::~eprop()
{
    if (nmolec > 0){
	delete[] pol_iso;
	delete[] pol_aniso;
    }
    
}

struct connect {
    inline connect();

    size_t nmolec;
    std::vector< std::vector<int> > partners;
    std::vector<int> central;
};

inline connect::connect()
: nmolec(0), partners(0), central(0)
{
}

} // namespace raman

#endif // STRUCT_RAMAN_TCC_H
