#ifndef STRUCT_SFG_H
#define STRUCT_SFG_H

#include <vector>
#include <string>
#include <iostream>

namespace sfg {

struct eprop {
    inline eprop();
    inline eprop(const eprop&);
    inline ~eprop();

    size_t nmolec;
    double* dip_z;  // nmolec
    double* pol_xx;  // nmolec
    double* gscale;  // nmolec
};

inline eprop::eprop()
: nmolec(0), dip_z(0), pol_xx(0), gscale(0)
{
}

inline eprop::eprop(const eprop& o)
{
    nmolec  = o.nmolec;
    if (nmolec > 0) {
	dip_z = new double[nmolec];
    	std::copy(o.dip_z, o.dip_z + nmolec, dip_z);
	pol_xx = new double[nmolec];
    	std::copy(o.pol_xx, o.pol_xx + nmolec, pol_xx);
	gscale = new double[nmolec];
    	std::copy(o.gscale, o.gscale + nmolec, gscale);
    }
}

inline eprop::~eprop()
{
    if (nmolec > 0){
	delete[] dip_z;
	delete[] pol_xx;
	delete[] gscale;
    }
    
}

struct connect {
    inline connect();

    size_t nmolec;
    std::vector< std::vector<int> > partners;
    std::vector<int> central;
};

inline connect::connect()
: nmolec(0), partners(0), central(0)
{
}

} // namespace sfg

#endif // STRUCT_SFG_H
