#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cstdlib>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <string>
#include <iostream>

#include <cmath>
#include <cassert>

#include "fgh.h"
#include "interpolation.h"
#include "constants.h"

#include "mat_tools.h"

//#include <limits>
//#include <gsl/gsl_matrix.h>
//#include <gsl/gsl_linalg.h>
//#include <gsl/gsl_blas.h>

namespace {

const double PI = M_PI;
const double au2kcal = 627.509;
const double au2ang = 0.529177249;

inline double harmonic(const double x, const double& L)
{
    return 0.5*std::pow(3.0*(x-L)/2.0, 2);
}

inline double morse(const double x)
{
    double dhart = 0.2250073497;
    double beta = 1.1741;
    double r0 = 1.7329;

    double ef = 1.0 - std::exp(-beta*(x-r0));

    return (dhart*ef*ef);
}

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(void)
{

    const double L = 7;
    size_t N = 14;

    double pot[N];
    std::fill(pot, pot + N, 0.0);

    const double V[] = {
	21802.24489,
	21538.64621,
	21410.57441,
	21352.78110,
	21331.08374,
	21327.71619,
	21333.33265,
	21342.92474,
	21353.78868,
	21364.47657,
	21374.22863,
	21382.66373,
	21389.60204,
	21394.94227};

    double x[] = {
         0.50000,
         0.60000,
         0.70000,
         0.80000,
         0.90000,
         1.00000,
         1.10000,
         1.20000,
         1.30000,
         1.40000,
         1.50000,
         1.60000,
         1.70000,
	 1.80000};

    for(size_t i = 0; i < N; ++i){
	pot[i] = V[i]/au2kcal;
	x[i] /= au2ang;
    }
    for(size_t i = 0; i < N; ++i){
	pot[i] = morse(x[i]);
    }
    const size_t nsample(128);
    const double dx = (x[N-1] - x[0])/nsample;

    double v_interpolated[nsample];

    for(size_t i = 0; i < nsample; ++i){
	double xq = dx*i + x[0];
	v_interpolated[i] = tools::cubic_spline(N, x, pot, xq);
    }

//    for(int i = 0; i < N; ++i)
//	std::cout << x[i] << "  " << pot[i] << std::endl;

    //double H[N*N];
    //std::fill(H, H + N*N, 0.0);
    //tools::cpt_H(N, x[N-1] - x[0], pot, H);

    double H[nsample*nsample];
    std::fill(H, H + nsample*nsample, 0.0);


    const double amu2au = 1822.88839;
    const double ma = 1.00794*amu2au; //H
    const double mb = 15.999*amu2au; //O
    const double reduced_mass = ma*mb/(ma+mb);

    tools::cpt_H(nsample, x[N-1] - x[0], v_interpolated, reduced_mass, H);
    std::cout << "completed cpt_H" << std::endl;

#if 0
    for(size_t i = 0; i < N; ++i){
	const size_t iN = i*N;
	for(size_t j = 0; j < N; ++j){
	    size_t ji = iN + j;
	    std::cout << H[ji] << "  ";
	}
	std::cout << std::endl;
    }
#endif

    //double eig[N];
    //tools::eigenvalues(N, H, eig);
    double eig[nsample];
    tools::eigenvalues(nsample, H, eig);
    //int err = tools::eigenvalues(N, H, 1000, 1e-6);

    //if(err != 0){
    //    std::cerr << "Failed to converge" << std::endl;
    //}

    std::cout << "\n\n";
    for(size_t i = 0; i < nsample; ++i)
	eig[i] *= constants::Eh_cm1;
    tools::print_m_by_n(nsample, 1, eig);

    //expected energy levels in cm-1
    // 1284.72267  3780.33105  6175.40710
    // \Delta \nu: 2495.60838  2395.07605

    //double euler[6];

    //ccpol::cart_to_euler(coord, euler);
    return 0;
}
