#include <cmath>

#include <fstream>
#include <iomanip>
#include <iostream>
#include <stdexcept>

#include "dip_poly2b5.h"
#include "alpha-el-poly2b3.h"

int main(int argc, char** argv)
{
    if (argc != 1) {
        std::cerr << "usage: test-mb-mu-dimer"
                  << std::endl;
        return 0;
    }

    double dimer[] = { -1.516074336e+00,  -2.023167650e-01,   1.454672917e+00,
                       -6.218989773e-01,  -6.009430735e-01,   1.572437625e+00,
                       -2.017613812e+00,  -4.190350349e-01,   2.239642849e+00,
                       -1.763651687e+00,  -3.816594649e-01,  -1.300353949e+00,
                       -1.903851736e+00,  -4.935677617e-01,  -3.457810126e-01,
                       -2.527904158e+00,  -7.613550077e-01,  -1.733803676e+00};

    std::cout << std::setprecision(9);

    double dip[3];
    h2o::dip_poly2b5::eval(dimer, dimer + 9, dip);

    std::cout << std::setprecision(6);
    std::cout << "Dipole (Debye): \n"
	      << std::setw(20) << dip[0]  
	      << std::setw(15) << dip[1]  
	      << std::setw(15) << dip[2] << std::endl;

    double alpha[9];
    h2o::alpha_el_poly2b3::eval(dimer, dimer + 9, alpha);

    std::cout << std::setprecision(6);
    std::cout << "Polarizability (Angstrom^3): \n";
    for(size_t i = 0; i < 3; ++i){
	for(size_t j = 0; j < 3; ++j){
	    std::cout << std::setw(20) << alpha[3*i + j];
	}
	std::cout << std::endl;
    }

    return 0;
}
