#ifndef ELECTROSTATICS_H
#define ELECTROSTATICS_H

#include <set>
#include <utility>
#include <cstddef>

#include "smear.h"

namespace ttm {

struct electrostatics {

    electrostatics();
    ~electrostatics();

    typedef std::set<std::pair<size_t, size_t> > excluded_set_type;

    // energy
    double operator()(size_t N,
                      const double* q,      // size=N, charge
                      const double* polfac, // size=N, damping factor
                      const double* polar,  // size=N, polarizability
                      const double* xyz,    // size=3N, coordinates
                      const excluded_set_type&,
                      const smear&,
                      double* grad = 0,
		      double aCC = 0, double aCD = 0, 
		      double aDD_inter = 0, double aDD_intra = 0);

    size_t size() const;

    // size=3*N, induced dipoles
    const double* dipoles() const;
    bool dipoles_converged() const;

    const double& E_ind() const { return m_Eind; }
    const double& E_elec() const { return m_Eelec; }
    const double* polarizability_tensor(size_t); // size = 9, row major

    // size=N (wrong unless gradients were requested)
    const double* phi() const;
    const double* phi_add() const;
    const double* phi_nonadd() const;
    double* dE_add();
    const double* dE_nonadd() const;

    static const size_t dip_maxiter = 5000;
    static const double dip_tolerance = 1.0e-15;

private:
    size_t  m_size;
    double* m_memory;

    void allocate(size_t);

private:
    bool m_converged;

private:
    double m_Eind, m_Eelec;

private:
    double m_pt[9];
};

} // namespace ttm

#endif // ELECTROSTATICS_H
