#include <cmath>
#include <cstdlib>
#include <iostream>

#include "fgh.h"

// Construct the Hamiltonian matrix H_ij according to Marston's
// "Fourier Grid Hamiltonian method"
//
// cmp. BALINT-KURTI et al., INTERNATIONAL REVIEWS IN PHYSICAL CHEMISTRY, VOL. 11, No. 2, 317-344
// (http://www.tandfonline.com/toc/trpc20/11/2)
//
// The implementations actually used are taken from D. Tannor's book
// "Introdoction to Quantum Mechanics: a time-dependent perspective"

namespace tools{

void cpt_H(const size_t& N, const double L, const double* V, 
	   const double red_mass, double* Hij)
{
    // compute the Hamiltonian NxN matrix in atomic units (hbar = 1; m = 1)

    const double K = M_PI*M_PI/(red_mass*(L*L));

    for(size_t i = 0; i < N; ++i){
	const size_t iN = i*N;
	for(size_t j = 0; j <= i; ++j){
	    if(i == j){
		Hij[iN + j] = K*((N-1.0)*(N-2.0)/6.0 + N/2.0) + V[i];
	    }else{
		Hij[iN + j] = std::pow(-1, (i-j)) * K
		            * std::pow((1.0/(std::sin(M_PI*(i-j)/N))), 2);
		Hij[j*N + i] = Hij[iN + j]; // Hermitian symmetry
	    }
	}
    }
}

}
