#ifndef FINITE_DIFFERENCES_H
#define FINITE_DIFFERENCES_H

#include <cmath>
#include <cstddef>

namespace ttm {

template <typename T>
inline void finite_differences
    (T& func, size_t Nw, double* x, double* df)
{
    const double EPS = 1.0e-5;

    for (size_t i = 0; i < 9*Nw; ++i) {
        const double orig = x[i];

        x[i] = orig + EPS;
        const double f2 = func(Nw, x);

        x[i] = orig - EPS;
        const double f1 = func(Nw, x);

        df[i] = 0.5*(f2 - f1)/EPS;
        x[i] = orig;
    }
}

template <typename T>
inline void finite_differences_damping
    (T& func, const size_t Nw, const double* xyz, double* df, 
     const double* param)
{
    const double EPS = 1.0e-5;

    double tmp_param[4];

    for (size_t i = 0; i < 4; ++i)
	tmp_param[i] = param[i];

    for (size_t i = 0; i < 4; ++i) {
        const double orig = tmp_param[i];

        tmp_param[i] = orig + EPS;
	func.set_new_damping(true,tmp_param); 
        const double f2 = func(Nw, xyz);

        tmp_param[i] = orig - EPS;
	func.set_new_damping(true,tmp_param); 
        const double f1 = func(Nw, xyz);

        df[i] = 0.5*(f2 - f1)/EPS;
        tmp_param[i] = orig;
    }
}

} // namespace ttm

#endif // FINITE_DIFFERENCES_H
