#ifdef HAVE_CONFIG_H
#   include "config.h"
#endif /* HAVE_CONFIG_H */

#include <cmath>
#include <cassert>

#include <algorithm>

//#include "macros.h"
#include "qtip4pf.h"
#include "constants.h"

//
// this is q-TIP4P/F : http://jcp.aip.org/resource/1/jcpsa6/v131/i2/p024501_s1
//

////////////////////////////////////////////////////////////////////////////////

namespace {

//----------------------------------------------------------------------------//

const double qM = -1.1128*constants::CHARGECON;
const double qH = -qM/2;

//----------------------------------------------------------------------------//

const double gammaM = 0.73612;
const double gamma1 = (1.0 - gammaM)/2;

//inline void compute_M_site_crd
//    (const double O[3], const double H1[3], const double H2[3],
//     double M[3])
//{
inline void compute_M_site_crd
    (const double* __restrict__ O,
     const double* __restrict__ H1,
     const double* __restrict__ H2,
     double* __restrict__ M)
{    
    for (size_t i = 0; i < 3; ++i)
        M[i] = gammaM*O[i] + gamma1*(H1[i] + H2[i]);
}

//----------------------------------------------------------------------------//

} // namespace

////////////////////////////////////////////////////////////////////////////////

namespace h2o {

//----------------------------------------------------------------------------//

qtip4pf::qtip4pf()
{
    m_nw = 0;
    m_mem = 0;
}

//----------------------------------------------------------------------------//

qtip4pf::~qtip4pf()
{
    if (m_nw > 0)
        delete[] m_mem;
}

//----------------------------------------------------------------------------//

void qtip4pf::allocate(size_t nw)
{
    if (m_nw > nw)
        return;

    delete[] m_mem;

    // see operator() below
    m_mem = new double[3*nw];

    m_nw = nw;
}

//----------------------------------------------------------------------------//

void   qtip4pf::operator()
   (size_t nw, const double* __restrict__ crd, double* __restrict__ dip)
{
    assert(crd || nw == 0);

    allocate(nw);

    // setup pointers
    double* msite = m_mem;

    // compute M-sites and 1-body terms
    for (size_t i = 0; i < nw; ++i) {
        const size_t i3 = 3*i;
        const size_t i9 = 3*i3;

        compute_M_site_crd(crd + i9, crd + i9 + 3, crd + i9 + 6, msite + i3);
    }

    std::fill(dip, dip + 3, 0.0);

    // permanent dipoles
    for (size_t n = 0; n < m_nw; ++n) {
        const size_t n3 = 3*n;
        const size_t n9 = 3*n3;

        dip[0] += qM*msite[n3 + 0] + qH*crd[n9 + 3] + qH*crd[n9 + 6];
        dip[1] += qM*msite[n3 + 1] + qH*crd[n9 + 4] + qH*crd[n9 + 7];
        dip[2] += qM*msite[n3 + 2] + qH*crd[n9 + 5] + qH*crd[n9 + 8];
    }

    for(size_t i = 0; i < 3; ++i)
	dip[i] *= constants::DEBYE/constants::CHARGECON;

}

//----------------------------------------------------------------------------//

} // namespace h2o

////////////////////////////////////////////////////////////////////////////////
