#ifndef READ_XYZ_H
#define READ_XYZ_H

#include <vector>
#include <iostream>
#include <string>

namespace tools {

struct xyz_frame {
    inline xyz_frame();
    inline xyz_frame(const xyz_frame&);
    inline ~xyz_frame();
    inline xyz_frame& operator=(const xyz_frame&);

    size_t nw;
    double* xyz; // 9*nw
    double* dip; // 3
    double* pol; // 6
};

// returns number of frames loaded
size_t read_xyz(const char*, std::vector<xyz_frame>&, 
		bool do_dip = false, bool do_pol = false);

void print_xyz(std::ostream&, const size_t&, const double*,std::string comment = "");
void swap_xyz(double*, double*);

inline xyz_frame::xyz_frame()
: nw(0), xyz(0), dip(0), pol(0)
{
}

inline xyz_frame::xyz_frame(const xyz_frame& o)
{
    nw = o.nw;
    if (nw > 0) {
        xyz = new double[9*nw];
        std::copy(o.xyz, o.xyz + 9*nw, xyz);
        dip = new double[3];
        std::copy(o.dip, o.dip + 3, dip);
        pol = new double[6];
        std::copy(o.pol, o.pol + 6, pol);
    }
}

inline xyz_frame::~xyz_frame()
{
    if (nw > 0)
        delete[] xyz;
        delete[] dip;
        delete[] pol;
}

inline xyz_frame& xyz_frame::operator=(const xyz_frame& o)
{
    if (this != &o) {
	if (nw > 0){
	    delete[] xyz;
            delete[] dip;
            delete[] pol;
	}

        nw = o.nw;
        if (nw > 0) {
            xyz = new double[9*nw];
            std::copy(o.xyz, o.xyz + 9*nw, xyz);
	    dip = new double[3];
	    std::copy(o.dip, o.dip + 3, dip);
	    pol = new double[6];
	    std::copy(o.pol, o.pol + 6, pol);
        }
    }

    return *this;
}

} // namespace tools

#endif // READ_XYZ_H
