#ifndef STRUCT_FRAME_H
#define STRUCT_FRAME_H

#include <vector>
//#include <string>
//#include <iostream>

#include "macros.h"

namespace opt {

struct hist_frame {
    inline hist_frame();
    inline hist_frame(const hist_frame&);
    inline ~hist_frame();
    inline hist_frame& operator=(const hist_frame&);

    size_t natm;
    double* xyz; // 3*natm
    char* name; // natm * MAX_CHAR_LENGTH
    int* atomic_num; // natm;
    double* mass; // natm
    double* cell;
    double* rcell;
    double* dipind;
    double* dipmol;
    double vol;
    size_t timestep;
    double time;
    size_t imcon;
};

inline hist_frame::hist_frame()
: natm(0), xyz(0), name(0), atomic_num(0), mass(0), cell(0), rcell(0),
    dipind(0), dipmol(0), vol(0), 
    timestep(0), time(0), imcon(0)
{
}

inline hist_frame::hist_frame(const hist_frame& o)
{
    natm = o.natm;
    if (natm > 0) {
        xyz = new double[3*natm];
        std::copy(o.xyz, o.xyz + 3*natm, xyz);
	name = new char[natm*MAX_CHAR_LENGTH];
    	std::copy(o.name, o.name + natm*MAX_CHAR_LENGTH, name);
	atomic_num = new int[natm];
    	std::copy(o.atomic_num, o.atomic_num + natm, atomic_num);
    	mass = new double[natm];
    	std::copy(o.mass, o.mass + natm, mass);
    	cell = new double[9];
    	std::copy(o.cell, o.cell + 9, cell);
    	rcell = new double[9];
    	std::copy(o.rcell, o.rcell + 9, rcell);
        dipind = new double[natm];
        std::copy(o.dipind, o.dipind + natm, dipind);
        dipmol = new double[natm];
        std::copy(o.dipmol, o.dipmol + natm, dipmol);
	vol = o.vol;
	timestep = o.timestep;
	time = o.time;
	imcon = o.imcon;
    }
}

inline hist_frame::~hist_frame()
{
    if (natm > 0){
     	delete[] xyz;
	delete[] name;
	delete[] atomic_num;
	delete[] mass;
	delete[] cell;
	delete[] rcell;
	delete[] dipind;
	delete[] dipmol;
    }
}

inline hist_frame& hist_frame::operator=(const hist_frame& o)
{
    if (this != &o) {
        if (natm > 0){
            delete[] xyz;
	    delete[] name;
            delete[] atomic_num;
            delete[] mass;
	    delete[] cell;
	    delete[] rcell;
	    delete[] dipind;
	    delete[] dipmol;
	}

        natm = o.natm;
        if (natm > 0) {
            xyz = new double[3*natm];
            std::copy(o.xyz, o.xyz + 3*natm, xyz);
	    name = new char[natm*MAX_CHAR_LENGTH];
	    std::copy(o.name, o.name + natm*MAX_CHAR_LENGTH, name);
            atomic_num = new int[natm];
            std::copy(o.atomic_num, o.atomic_num + natm, atomic_num);
            mass = new double[natm];
            std::copy(o.mass, o.mass + natm, mass);
	    cell = new double[9];
	    std::copy(o.cell, o.cell + 9, cell);
	    rcell = new double[9];
	    std::copy(o.rcell, o.rcell + 9, rcell);
	    dipind = new double[natm];
	    std::copy(o.dipind, o.dipind + natm, dipind);
	    dipmol = new double[natm];
	    std::copy(o.dipmol, o.dipmol + natm, dipmol);
	    vol = o.vol;
	    timestep = o.timestep;
	    time = o.time;
	    imcon = o.imcon;
	}
    }

    return *this;
}

} // namespace opt

#endif // STRUCT_FRAME_H
