#ifndef STRUCT_PAIR_FRAME_H
#define STRUCT_PAIR_FRAME_H

namespace tools {

struct pair_frame {
    inline pair_frame();
    inline pair_frame(const int&);
    inline pair_frame(const pair_frame&);
    inline ~pair_frame();

    size_t npairs;
    double time;
    double* d_rij;  // 3 * npairs
    int* region;    //     npairs
};

inline pair_frame::pair_frame()
: npairs(0), time(0), d_rij(0), region(0)
{
}

inline pair_frame::pair_frame(const int& frame_npairs)
{
    npairs  = frame_npairs;
    time    = 0.0;
    if (npairs > 0) {
	d_rij = new double[3*npairs];
	region = new int[npairs];
    }
}

inline pair_frame::pair_frame(const pair_frame& o)
{
    npairs  = o.npairs;
    time    = o.time;
    if (npairs > 0) {
	d_rij = new double[3*npairs];
    	std::copy(o.d_rij, o.d_rij + 3*npairs, d_rij);
	region = new int[npairs];
    	std::copy(o.region, o.region + npairs, region);
    }
}

inline pair_frame::~pair_frame()
{
    if (npairs > 0){
	delete[] d_rij;
	delete[] region;
    }
    
}

} // namespace tools

#endif // STRUCT_PAIR_FRAME_H
