#include <cmath>

#include <iostream>

#include "ltp2011.h"

int main(int argc, char** argv)
{
    h2o::ltp2011 dip_model;

    std::cout << "Molecule orientation:\n"
	      << "  Molecule in xy plane, with HOH bisector on x-axis\n"
	      << "  O at origin, H1 is in quadrant 1, H2 in quadrant 4\n\n";

    std::cout << "Expected output for R_1 = R_2 = 0.96A and 104.5 degrees:\n"
	      << "mu_x = 1.8521032323243665  Debye" << std::endl
	      << "mu_y = 0.0000000000000000  Debye" << std::endl << std::endl;

    const double R(0.96);
    double cos_th(std::cos(104.5/180*M_PI));

    double mu_x(-1);
    double mu_y(-1);

    dip_model(R, R, cos_th, mu_x, mu_y);

    std::cout << "c++ side: " << std::endl
	      << "mu_x = " << mu_x << std::endl
	      << "mu_y = " << mu_y << std::endl;

    std::cout << std::endl;
    std::cout << "Expected output for R_1 = 0.92, R_2 = 1.01A and 109.5 degrees:\n"
	      << "mu_x =  1.77849933145      Debye" << std::endl
	      << "mu_y = -7.78998668143E-002 Debye" << std::endl << std::endl;

    double R1(0.92);
    double R2(1.01);
    cos_th = std::cos(109.5/180*M_PI);

    mu_x = -1;
    mu_y = -1;

    dip_model(R1, R2, cos_th, mu_x, mu_y);

    std::cout << "c++ side: " << std::endl
	      << "mu_x = " << mu_x << std::endl
	      << "mu_y = " << mu_y << std::endl;

    R1 = 1.01;
    R2 = 0.92;

    mu_x = -1;
    mu_y = -1;

    std::cout << "\nChecking for permutational invariance. "
              << "sign of mu_y should switch" << std::endl;
    std::cout << "for R_1 = 1.01, R_2 = 0.92A and 109.5 degrees:" << std::endl;
    dip_model(R1, R2, cos_th, mu_x, mu_y);

    std::cout << "c++ side: " << std::endl
	      << "mu_x = " << mu_x << std::endl
	      << "mu_y = " << mu_y << std::endl;

}
