#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>

#include <iomanip>
#include <iostream>
#include <fstream>
#include <stdexcept>

#include "read-xyz.h"

#include "mbpol.h"

namespace {

static std::vector<tools::xyz_frame> molec;

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{

    try {
	std::ifstream ifs(argv[1]);

	if (!ifs)
	    throw std::runtime_error("could not open the XYZ file");

	std::string comment;
	tools::read_xyz(argv[1], molec, false);
    } catch (const std::exception& e) {
	std::cerr << " ** Error ** : " << e.what() << std::endl;
	return 1;
    }

    x2o::mbpol pot;

    const size_t nw = molec[0].nw;

    double grd[9*nw];
    double E = pot(nw, molec[0].xyz, grd);

    std::cout << "       E = " << E << "\n"
	<< "E[nogrd] = " << pot(nw, molec[0].xyz) << "\n"
	<< std::endl;

    const double eps = 1.0e-4;
    double crd[9*nw];
    std::copy(molec[0].xyz, molec[0].xyz + 9*nw, crd);

    for (int n = 0; n < 9*nw; ++n) {
	double tmp[9*nw];
	const double x_orig = crd[n];

	crd[n] = x_orig + eps;
//	const double Ep = pot(nw, &(crd[0]), tmp);
	const double Ep = pot(nw, crd, tmp);

	crd[n] = x_orig - eps;
	const double Em = pot(nw, crd, tmp);
//	const double Em = pot(nw, &(crd[0]), tmp);

	const double gfd = 0.5*(Ep - Em)/eps;
	crd[n] = x_orig;

	std::cout << n << ' ' << std::fabs(grd[n] - gfd) << '\n';
    }

    return 0;
}
