#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>

#include <sstream>
#include <iomanip>
#include <iostream>
#include <fstream>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit.h>

#include "tools/ps.h"

#include "read-xyz.h"

int main(int argc, char** argv)
{
    // 1. load reference trimer geometries and binding energies

    if (argc < 2) {
   	std::cerr << "usage: fit-dip configs.xyz"<< std::endl;
   	return 0;
    }

    if (argv[1][0] == '\0') {
    	std::cerr << " ** Error ** : argv[1] is too short" << std::endl;
    	return 1;
    }

    std::vector<tools::xyz_frame> training_set;

    size_t k_count = 0;
    try {
	while (--argc != 0) {
	    ++argv;
	    if ((*argv)[0] == '\0') {
	 	std::cerr << " ** Error ** : argument is too short"
	 	    << std::endl;
	  	return EXIT_FAILURE;
	    }
	    std::string arg = *argv;
	    size_t found_xyz = arg.find(".xyz");

    	    std::vector<tools::xyz_frame> this_file;
    	    tools::read_xyz(*argv, this_file, false);
    	    std::copy(this_file.begin(), this_file.end(),
	 	      std::back_inserter(training_set));

	    std::cerr << *argv << " : " << this_file.size() << std::endl;
	}
    } catch (const std::exception& e) {
	std::cerr << e.what() << std::endl;
	return EXIT_FAILURE;
    }

    // 2. go over the ref configurations and subtract the energy
    //    that we do not fit (i.e., monomer, electrostatic and 1/R^6)

    {
	std::ofstream outfile;
    	outfile.open("ps.test.xyz");

        for (size_t n = 0; n < training_set.size(); ++n) {

            const double* xyz = training_set[n].xyz;
	    const double Eps = ps::pot_nasa(xyz, 0);

	    std::stringstream ss;
	    ss << Eps;
	    std::string dipstring = ss.str();

	    std::cout << Eps << std::endl;


	    tools::print_xyz(outfile, 1, xyz, dipstring);

	}

	outfile.close();

    }

}
