#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif // HAVE_CONFIG_H

#include <cmath>
#include <cassert>

#include <iomanip>
#include <iostream>
#include <fstream>
#include <stdexcept>

#include "read-xyz.h"

#include "ttm4-dip.h"
#include "ttm4-es.h"

namespace {

static std::vector<tools::xyz_frame> molec;

} // namespace

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{

    try {
	std::ifstream ifs(argv[1]);

	if (!ifs)
	    throw std::runtime_error("could not open the XYZ file");

	std::string comment;
	tools::read_xyz(argv[1], molec, false);
    } catch (const std::exception& e) {
	std::cerr << " ** Error ** : " << e.what() << std::endl;
	return 1;
    }

    std::cout.precision(9);
    std::cout.setf(std::ios_base::showpoint);

    ttm::ttm4_dip dip;

    double dip_mb[3] = {0.0, 0.0, 0.0};

    dip(molec[0].nw, molec[0].xyz, dip_mb);

    double dip_ttm4[3] = {0.0, 0.0, 0.0};

    ttm::ttm4_es pot;
    double Eelec(0), Eind(0);
    pot(molec[0].nw, molec[0].xyz, Eelec, 0, Eind, 0);

    pot.dipind(molec[0].nw, dip_ttm4);

    std::cout << "Dipole\n"
	      << "   TTM4-F   |    MB-dip\n"
	      << dip_ttm4[0] << "     " << dip_mb[0] << "\n"
	      << dip_ttm4[1] << "     " << dip_mb[1] << "\n"
	      << dip_ttm4[2] << "     " << dip_mb[2] << "\n" << std::endl;

    return 0;
}
