#ifndef TTM4F_H
#define TTM4F_H

#include <cstddef>
#include "electrostatics.h"

namespace ttm {

struct ttm4f {

    // M-site positioning
    static const double gammaM;

    ttm4f();
    ~ttm4f();

    void set_vdw(bool);
    void set_calc_mon(bool);
    void set_new_damping(bool, double* a = 0);
    const double& gamma() const { return gammaM; }

    // energy
    double operator()(size_t, const double*, double* grad = 0); // O H H O H H

    const double& Evdw() const {return m_vdw; }
    const double& Eint() const {return m_int; }

    const double& Eind() const { return m_electrostatics.E_ind(); }
    const double& Eelec() const { return m_electrostatics.E_elec(); }

    const double* grad_add() const;
    const double* grad_nonadd() const;

    const double* pt(size_t nw)
        { return m_electrostatics.polarizability_tensor(4*nw); }

    void dip(double*, double* q = 0, double* pass_xyz = 0, 
	     double* moldipind = 0) const;

private:
    //flags
    bool calc_mon, calc_vdw, change_damping;
    // vdW
    double A6, A8, A10, A12, A14, A16;
    double aCC, aCD, aDD_inter, aDD_intra;

    // polarizabilities
    double polfacO, polfacH, polfacM;
    double polarO, polarH, polarM;

    double m_vdw, m_int;

private:
    void allocate(size_t);

    size_t  m_nmol;
    double* m_memory;

    electrostatics m_electrostatics;
    electrostatics::excluded_set_type m_excluded;

private: // non-copyable
    ttm4f(const ttm4f&);
    ttm4f& operator=(const ttm4f&);
};

} // namespace ttm

#endif // TTM4F_H
